#include <Arduino.h>
#include <U8g2lib.h>
//#include <SPI.h>
#include "NTPClient.h"
#include <ESP8266WiFi.h>
#include <WiFiUdp.h>
#include <Wire.h>
#include <RTClib.h>                                      // Подключаем библиотеку iarduino_RTC для работы с модулями реального времени.
  RTC_DS1307 rtc;                                        // Объявляем объект rtc для работы с RTC модулем на базе чипа DS1307/DS3231 по шине I2C

#include <Adafruit_BME280.h>                             // Подключаем библиотеку для работы с датчиком BME280
  Adafruit_BME280 sensor;                                // Объявляем объект sensor для работы с датчиком BME280, указываем его реальный адрес

const char *ssid     = "RT-GPON-E1E8";
const char *password = "221119661644545";

WiFiUDP ntpUDP;
NTPClient timeClient(ntpUDP);
U8G2_ST7920_128X64_F_HW_SPI u8g2(U8G2_R0, /* CS=*/ 15, /* reset=*/ 16); // Feather HUZZAH ESP8266, E=clock=14, RW=data=13, RS=CS=15

unsigned long timing;
float TimeDisp[3];                                      // Объявляем массив для хранения значений давления

void setup(){
 delay(300);                                            // Ждем готовности модулей отвечать на запросы
 Serial.begin(115200);                                  // Инициализируем монитор порта
 Wire.begin();
 
 rtc.begin();
  if (! rtc.isrunning()) {
    Serial.println("RTC is NOT running!");
    // Первоначальные установки модуля RTC (только при начальной компиляции) на всякий случай
    rtc.adjust(DateTime(__DATE__, __TIME__));
  }

 if (!sensor.begin()) {  
    Serial.println(F("Could not find a valid BME280 sensor, check wiring!"));
    while (1);
  }
  
 u8g2.begin();                                        // Инициализация графической библиотеки

 WiFi.begin(ssid, password);                           // Коннектимся к установленной сети WiFi 
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  timeClient.begin();                                             // Инициализируем NTPClient для получения времени
  timeClient.setTimeOffset(10800);

  timing = millis();                                              // Устанавливаем начальное значение переменной timing
    TimeDisp[0] = 0;                                              // Забиваем массив начальными значениями
    TimeDisp[1] = 0;
    TimeDisp[2] = 0;

  TimeCorr();                                                     // Коррекция установленного времени по интернету
}

void loop() {

  DateTime now = rtc.now();                                        // Получаем данные даты и времени из модуля RTC
//  sensor.read(1);                                                   // Получаем данные датчика BMP280 c типом давления в мм.рт.ст.
  Press();
      
  u8g2.firstPage();                                                // Вывод информации на дисплей в цикле: u8g.firstPage(); do{...команды...while(u8g.nextPage());

   do {  u8g2.setColorIndex(1);                                    // Выбираем черный цвет
         u8g2.drawBox(0, 0, 128, 11);                              // Выводим прямоугольник с координатами левого верхнего угла 0,0 и размерами 128x11
         u8g2.setFont(u8g2_font_6x10_tr);                          // Выбираем шрифт u8g2_font_6x10_tr
         u8g2.setColorIndex(0);                                    // Выбираем белый цвет (цвет фона)
         u8g2.drawStr(33, 9, "RN1TU Clock");                       // Выводим текст "RN1TU Clock"  в позиции  33х9
         
         u8g2.setColorIndex(1);                                    // Выбираем черный цвет
                 
         u8g2.setFont(u8g2_font_fub20_tn);                         // Выбираем шрифт u8g2_font_fub20_tn
         
         u8g2.setCursor(  3, 37); u8g2.print(now.hour()/10);       // Выводим старший разряд часов    в позиции   3х37
         u8g2.setCursor( 20, 37); u8g2.print(now.hour()%10);       // Выводим младший разряд часов    в позиции  20х37
         u8g2.drawStr  ( 34, 37, ":");                             // Выводим двоеточие               в позиции  33х37
         u8g2.setCursor( 46, 37); u8g2.print(now.minute()/10);     // Выводим старший разряд минут    в позиции  46х37
         u8g2.setCursor( 63, 37); u8g2.print(now.minute()%10);     // Выводим младший разряд минут    в позиции  63х37
         u8g2.drawStr  ( 77, 37, ":");                             // Выводим двоеточие               в позиции  76х37
         u8g2.setCursor( 89, 37); u8g2.print(now.second()/10);     // Выводим старший разряд секунд   в позиции  89х37
         u8g2.setCursor(106, 37); u8g2.print(now.second()%10);     // Выводим младший разряд секунд   в позиции 106х37
           
         u8g2.setFont(u8g2_font_6x10_tr);                          // Выбираем шрифт u8g2_font_6x10_tr

         u8g2.setCursor( 29, 50); u8g2.print(now.day());           // Выводим значение даты из модуля чаос реального времени  
         u8g2.drawStr  ( 45, 50, "-");                             // Выводим текстовую строку "-"
         u8g2.setCursor( 55, 50); u8g2.print(now.month());         // Выводим значение месяца из модуля чаос реального времени
         u8g2.drawStr  ( 70, 50, "-");                             // Выводим текстовую строку "-"
         u8g2.setCursor( 80, 50); u8g2.print(now.year());          // Выводим значение года из модуля часов реального времени

         u8g2.drawStr  (20, 62, "mmHg");                             // Выводим текстовую строку "mmHg"
         u8g2.setCursor( 0, 62); u8g2.print(sensor.readPressure()/133.3224,0);      // Выводим значение давления из модуля ВМР280 без знаков после запятой
         u8g2.drawStr  (92, 62, "C");                               // Выводим текстовую строку "C"
         u8g2.setCursor( 65, 62); u8g2.print(sensor.readTemperature(),1);  // Выводим значение температуры из модуля ВМР280 с одним знаком после запятой
         u8g2.drawUTF8  (123, 62, "%");                                // Выводим текстовую строку "%"
         u8g2.setCursor( 108, 62); u8g2.print(sensor.readHumidity(),0);      // Выводим значение давления из модуля ВМР280 без знаков после запятой

          
        if (TimeDisp[2] > TimeDisp[1] & TimeDisp[1] > TimeDisp[0]){  // Условие вывода обозначения роста давления
            u8g2.drawStr(47, 62, "+");                               // Вывод символа роста давления
        } 
         else if (TimeDisp[2] < TimeDisp[1] & TimeDisp[1] < TimeDisp[0]) {      // Условие вывода обозначения падения давления
            u8g2.drawStr(47, 62, "-");                               // Вывод символа падения давления
        } 
         else { u8g2.drawUTF8(48, 62, "~"); }                        // Условие и вывод символа неустойчивой тенденции изменения давления
         
      }
    while(u8g2.nextPage());
   
  delay(1000);
}

void TimeCorr() {                                                 // Функция получения данных времени от сервера NTP

  DateTime now = rtc.now();                                        // Получаем данные даты и времени из модуля RTC

// Получаем время с сервера и выводим в монитор порта
  while(!timeClient.update()) {
  timeClient.forceUpdate();
  }
  String formattedTime = timeClient.getFormattedTime();           // Получаем и печатаем в порт формализованное значение времени
  Serial.println(formattedTime);

// Извлекаем время и конвертируем в переменные типа int
  int NowHour = timeClient.getHours();
  int NowMin = timeClient.getMinutes();
  int NowSec = timeClient.getSeconds();
  int NowYear = timeClient.getYear(); 
  int NowMonth = timeClient.getMonth(); 
  int NowDate = timeClient.getDate(); 
    
  rtc.adjust(DateTime( NowYear, NowMonth, NowDate, NowHour, NowMin, NowSec));     // Коррекция времени в RTC модуле
}

void Press() {                                                    // Функция формирования массива динамики изменения давления
if (millis() - timing > 2500000){                                 // Условие внесение данных давления в массив по времени
  
   TimeDisp[0] = TimeDisp[1];                                     // Занесение в массив значений со смещением по истечении времени счета
   TimeDisp[1] = TimeDisp[2];
   TimeDisp[2] = sensor.readPressure();
   timing = millis();                                             // Сброс тайминга для следующего отсчета
  }

if (millis() - timing < 0){                                       // Проверка сброса функции millis() в начальное состояние
   timing = millis();                                             // Сброс тайминга для восстановления функциональности
  }
}
